/* testing TestRGF.java, Rony G. Flatscher, 2001-04-27, 2003-02-09, 2003-05-10, 2005-06-05, 2005-07-07

   purpose: this Rexx program demonstrates the usage of the Java class "TestRGF"
            and demonstrates the usage of the BSF-interface, concentrating on
            accessing arrays

   invoke:  enter the following command to inoke the Rexx program "testArrays2-oo.rex":

                  "java com.ibm.bsf.Main -lang rexx -in testArrays2-oo.rex"
            or
                  "java com.ibm.bsf.Main -in testArrays2-oo.rex"

            if extension "rex" is defined in the text file "\com\ibm\bsf\Languages.properties",
            which allows for determining the needed engine by looking at the file extension
*/


   /* create a one-dimensional Java array object */
say "1a) Demonstrating the creation and usage of a one dimensional Java array object of type 'int':"
arr=bsf.createArray("int.class", 3)    /* cf. reference card, note lower-case 'i' ! */
do i=1 to 3
   arr[i]=i
end

say "    elements in the Java array of type 'int' (primitive):"
say
do i=1 to arr~items
   say "    i="i":" arr[i]
end
say

say "1b) Demonstrating the creation and usage of a one dimensional Java array object of type 'java.lang.Integer':"
arr=bsf.createArray("Integer.class", 3)    /* cf. reference card, note upper-case 'I' ! */
do i=1 to 3
   arr[i]=box('int', i)    /* turn primitive 'int' type into a the Java object of type 'java.lang.Integer' */
end

say "    elements in the Java array of type 'java.lang.Integer' (Java objects):"
say
do i=1 to arr~items
   say "    i="i":" arr[i]
end
say



   /* create a two-dimensional Java array object */
say "2a) Demonstrating the creation and usage of a two dimensional Java array object of type 'float':"
arr=bsf.createArray("float.class", 3, 2)  /* cf. reference card, note lower-case 'f' ! */
do j=1 to 3
   do k=1 to 2
      arr[j,k]=j"."k    -- create a float (small decimal) value
   end
end

say "    elements in the Java array of type 'float' (primitive):"
say
do j=1 to 3
   do k=1 to 2
      say "    j="j",k="k":" arr[j,k]   -- retrieve and show the float value stored in the array
   end
end
say


say "2b) Demonstrating the creation and usage of a one dimensional Java array object of type 'java.lang.Float':"
arr=bsf.createArray("Float.class", 3,2)    /* cf. reference card, note upper-case 'F' ! */
do j=1 to 3
   do k=1 to 2
      arr[j,k]=box("float", j"."k)  -- create a float (small decimal) value, box it as a "java.lang.Float" object
   end
end

say "    elements in the Java array of type 'java.lang.Float' (Java objects):"
say
do j=1 to 3
   do k=1 to 2
      say "    j="j",k="k":" arr[j,k]   -- retrieve and show the float value stored in the array
   end
end
say



say "3)  Demonstrating the creation and usage of a one dimensional Java array object of type 'java.lang.String':"
arr=bsf.createArray("String.class", 3)      /* cf. reference card, note upper-case 'F' ! */
arr[1]="Hi, there"
arr[2]="this is ooRexx"
arr[3]="using Java array objects to store and fetch Strings!"

say
do i=1 to arr~items
   say "    i="i":" arr[i]
end
say






say "////////////////"




arr=bsf.createArray("int.class", 3)
do i=1 to 3
   arr[i]=i
end

do i over arr
   say i
end

arr~putStrict("int",9,1)      -- o.k.
arr~putStrict("in",10)       -- too few scubscripts
arr~putStrict("i",11,1,2)   -- too many subscripts

arr~put(9,1)      -- o.k.
arr~put(11,1,2)   -- too many subscripts
arr~put(10)       -- too few scubscripts

say "arr~at(1):" pp(arr~at(1))
say "arr~at(1,2):" pp(arr~at(1,2))
say "arr~at:" pp(arr~at)

say "arr~at(.array~of(1)):" pp(arr~at(.array~of(1)))
say "arr~at(.array~new):" pp(arr~at(.array~of(9,3)))
say "arr~at(.array~new):" pp(arr~at(.array~new))

say "arr~at(.array~new):" pp(arr~at(.array~new(2,3)~~put(19,1,1) ))





::requires BSF.CLS    -- get access to the BSF-supporting Object Rexx classes

