// ---rgf, 2008-08-13
// import com.ibm.bsf.*;   // BSF support
import org.apache.bsf.*;   // BSF support

import org.rexxla.bsf.engines.rexx.RexxEngine;  // rgf, 2008-08-13, needed for casting!
import java.util.Vector;

import java.io.*;       // exception handling

/** Test Java program which employs dispatching Rexx invocations on
 *  its own thread.
 * @author   Rony G. Flatscher, 2008-08-13
 */

public class TestAccBSFRegistry {

  /** Running an in-line defined Rexx script. */
  public static void main (String[] args) throws IOException
  {
    try
    {

      BSFManager mgr  = new BSFManager ();

      BSFEngine  rexx = mgr.loadScriptingEngine("rexx");
      // mgr.declareBean("iB", new Integer(0), int.class); // define an entry in BSFRegistry
      mgr.registerBean("iB", new Integer(0)); // define an entry in BSFRegistry

      String     rexxCode[]=new String [3];
      rexxCode[0] = "SAY '   hint ... TID:' bsfGetTID()~right(8)',  eval(): Rexx was here! iB='bsf('lookupBean','iB') || ', adding 1 to' bsf('registerBean', 'iB', 'java.lang.Integer', bsf('lookupBean', 'iB')+1) '...'";
      rexxCode[1] = "SAY '   hint ... TID:' bsfGetTID()~right(8)', apply(): Rexx was here! iB='bsf('lookupBean','iB') || ', adding 1 to' bsf('registerBean', 'iB', 'java.lang.Integer', bsf('lookupBean', 'iB')+1) '...'";
      rexxCode[2] = "SAY '   hint ... TID:' bsfGetTID()~right(8)', apply():' 'arg()='arg()', arg1=['arg(1)'], arg2='||('['arg(2)']')~left(6)': iB='bsf('lookupBean','iB') || ', adding 1 to' bsf('registerBean', 'iB', 'java.lang.Integer', bsf('lookupBean', 'iB')+1) '...'";

      String hint[] = {"<---NESTED--->",
                       "<--separate-->"  };


      int        reps = 5; // total number of repetitions


      System.out.println(TestDispatchingOnOwnJavaThread.class.getName()+": RexxEngine.version=["+((RexxEngine) rexx).version+"]\n");

      // ------------------------------------------------------------------------------------
      ((RexxEngine) rexx).setThreadedRexxStart(false);  // new in RexxEngine as of 2008-08-13
      System.out.println("--- (1a): eval(), dispatching scripts on SAME thread (nesting) ---");
                // invoke Rexx and retrieve its return value
      for (int i=0;i<reps;i++)
      {
          rexx.eval ("rexx", 0, 0, rexxCode[0].replaceFirst("hint", hint[0]));
      }
      System.out.println("\n--- (1b): eval(), dispatching scripts on separate threads ---");

      ((RexxEngine) rexx).setThreadedRexxStart(true);  // new in RexxEngine as of 2008-08-13
      for (int i=0;i<reps;i++)
      {
          rexx.eval ("rexx", 0, 0, rexxCode[0].replaceFirst("hint", hint[1]));
      }
      System.out.println("-------------------------------------------------------------\n");


      // ------------------------------------------------------------------------------------
      System.out.println("--- (2a): apply(), dispatching scripts on SAME thread (nesting) ---");

      // System.out.println("--- begin of apply(), followed by call(): no args, though ---");
      ((RexxEngine) rexx).setThreadedRexxStart(false);  // new in RexxEngine as of 2008-08-13
      for (int i=0;i<reps;i++)
      {
          rexx.apply ("rexx", 0, 0, rexxCode[1].replaceFirst("hint", hint[0]), null, null);
      }
      System.out.println("\n--- (2b): apply()+call(), dispatching scripts on separate threads ---");

      ((RexxEngine) rexx).setThreadedRexxStart(true);  // new in RexxEngine as of 2008-08-13
      for (int i=0;i<reps;i++)
      {
          rexx.apply ("rexx", 0, 0, rexxCode[1].replaceFirst("hint", hint[1]), null, null);
      }
      System.out.println("-------------------------------------------------------------\n");


      // ------------------------------------------------------------------------------------
      System.out.println("--- (3a): begin of apply(): WITH args, dispatching scripts on SAME thread (nesting) ---");
      ((RexxEngine) rexx).setThreadedRexxStart(false);  // new in RexxEngine as of 2008-08-13

      Vector v=new Vector(2);
      v.addElement("one");
      v.addElement(null);

      Object o[]={"ONE", "NIXI"};

      for (int i=0;i<reps;i++)
      {
          rexx.apply ("rexx", 0, 0, rexxCode[2].replaceFirst("hint", hint[0]), null, v);
      }
      System.out.println("\n--- (3b): apply(), dispatching scripts on separate threads ---");

      ((RexxEngine) rexx).setThreadedRexxStart(true);  // new in RexxEngine as of 2008-08-13
      for (int i=0;i<reps;i++)
      {
          rexx.apply ("rexx", 0, 0, rexxCode[2].replaceFirst("hint", hint[1]), null, v);
          rexx.call (null, null, o);
      }
      System.out.println("-------------------------------------------------------------\n");

    }
    catch (BSFException e)
    {
      e.printStackTrace();
    }
  }
}

