// ---rgf, 2008-08-13
// import com.ibm.bsf.*;   // BSF support
import org.apache.bsf.*;   // BSF support

import org.rexxla.bsf.engines.rexx.RexxEngine;  // rgf, 2008-08-13, needed for casting!
import java.util.Vector;

import java.io.*;       // exception handling

/** Test Java program which employs dispatching Rexx invocations on
 *  its own thread.
 * @author   Rony G. Flatscher, 2008-08-13
 */

public class TestDispatchingOnOwnJavaThread {

  /** Running an in-line defined Rexx script. */
  public static void main (String[] args) throws IOException
  {
    try
    {

      BSFManager mgr  = new BSFManager ();
      BSFEngine  rexx = mgr.loadScriptingEngine("rexx");
      String     rexxCode = "SAY '   eval(): Rexx was here! (same Java thread):' bsfGetTID()";
      int        reps = 3; // total number of repetitions


      System.out.println(TestDispatchingOnOwnJavaThread.class.getName()+": RexxEngine.version=["+((RexxEngine) rexx).version+"]");

      ((RexxEngine) rexx).setThreadedRexxStart(false);  // new in RexxEngine as of 2008-08-13
      System.out.println("--- (1a): eval(), dispatching scripts on SAME thread (nesting) ---");
                // invoke Rexx and retrieve its return value
      rexx.eval ("rexx", 0, 0, rexxCode);
      System.out.println("---\n");
      System.out.println("--- (1b): eval(), dispatching scripts on separate threads ---");

      ((RexxEngine) rexx).setThreadedRexxStart(true);  // new in RexxEngine as of 2008-08-13
      rexxCode = "SAY '   eval(): Rexx was here! (executing on a NEW Java thread):'  bsfGetTID()";
      for (int i=0;i<reps;i++)
      {
          rexx.eval ("rexx", 0, 0, rexxCode);
      }
      System.out.println("--- (1c): end of eval() ---");
      System.out.println("-------------------------------------------------------------\n");


      // ------------------------------------------------------------------------------------
      System.out.println("--- (2a): apply(), dispatching scripts on SAME thread (nesting) ---");
      rexxCode = "SAY '   apply(): Rexx was here! (same Java thread):' bsfGetTID()";

      // System.out.println("--- begin of apply(), followed by call(): no args, though ---");
      ((RexxEngine) rexx).setThreadedRexxStart(false);  // new in RexxEngine as of 2008-08-13
      rexx.apply ("rexx", 0, 0, rexxCode, null, null);
      System.out.println("---\n");
      System.out.println("--- (2b): apply()+call(), dispatching scripts on separate threads ---");

      ((RexxEngine) rexx).setThreadedRexxStart(true);  // new in RexxEngine as of 2008-08-13
      rexxCode = "SAY '   apply(): Rexx was here! (executing on a NEW Java thread):'  bsfGetTID()";
      for (int i=0;i<reps;i++)
      {
          rexx.apply ("rexx", 0, 0, rexxCode, null, null);
      }
      System.out.println("--- (2c): end of apply() ---");
      System.out.println("-------------------------------------------------------------\n");


      // ------------------------------------------------------------------------------------
      System.out.println("--- (3a): begin of apply(): WITH args, dispatching scripts on SAME thread (nesting) ---");
      rexxCode = "SAY '   apply():' 'arg1=['arg(1)']/arg2=['arg(2)']/arg3=['arg(3)']: Rexx was here! (executing on a NEW Java thread):'  bsfGetTID()";
      ((RexxEngine) rexx).setThreadedRexxStart(false);  // new in RexxEngine as of 2008-08-13

      Vector v=new Vector(4);
      v.addElement("one");
      v.addElement("deux");
      v.addElement("drei");
      v.addElement("quattro");

      Object o[]={"-eins", "two", "tre", "quattre-"};

      rexx.apply ("rexx", 0, 0, rexxCode, null, v);
      System.out.println("---\n");
      System.out.println("--- (3b): apply(), dispatching scripts on separate threads ---");

      ((RexxEngine) rexx).setThreadedRexxStart(true);  // new in RexxEngine as of 2008-08-13
      for (int i=0;i<reps;i++)
      {
          rexx.apply ("rexx", 0, 0, rexxCode, null, v);
      }
      System.out.println("--- (3c): end of apply(): WITH args ---");
      System.out.println("-------------------------------------------------------------\n");

    }
    catch (BSFException e)
    {
      e.printStackTrace();
    }
  }
}

