// rgf, 2008-08-24: test execution of Rexx scripts by concurrently
//                  creating and using Rexx engines



import org.apache.bsf.*;    // BSF support
import java.io.IOException; // exception handling

import java.io.File;
import java.io.FileReader;
import java.util.Vector;

/** Test Java program which demonstrates how easy it is to invoke Rexx via BSF.
 * @author   Rony G. Flatscher, 2001-04-27, 2007-09-10
 */
public class TestNestedEngines {

  /** Running an in-line defined Rexx script. */
  public static void main (String[] args) throws Exception
  {
    try
    {

      String     rexxCode = null;   // Rexx code
      File       file=null;         // file object to use

      if (args.length>0)            // Rexx-file name supplied ?
      {
          file=new File(args[0]);   // create fileObject
      }
      else                          // use default Rexx file by the same name as this Java class
      {
          file=new File(TestNestedEngines.class.getName()+".rex");
      }

// System.out.println("file=["+file+"]");
      rexxCode=(String) org.apache.bsf.util.IOUtils.getStringFromReader(new FileReader(args[0]));

        // execute the Rexx script
      String threadName=Thread.currentThread().getName();
      Vector vargs= new Vector(2);
      vargs.addElement(threadName);
      vargs.addElement("3");             // nr. of seconds to sellep

      // if (true==false)   // do not enter block (just uncomment to allow again)
      {
          BSFManager mgr  = new BSFManager ();               // get a BSFManager instance
          BSFEngine  rexx = mgr.loadScriptingEngine("rexx"); // load the Rexx engine for this BSFManager instance

          String result=(String) rexx.apply ("rexx", 0, 0,
                             (Object) rexxCode.replaceAll("@threadName", (""+threadName)),
                             null,
                             vargs);

          System.out.println("--- Java/"+threadName+": received result from Rexx=["+result+"]\n");
          System.out.flush();
      }


      int nrThreads=1;

      if (args.length>1)    // arg # 2, if given, is number of threads
      {
          System.out.println("Nr. of threads to use supplied as arg # 2=["+args[1]+"]");
          nrThreads=new Integer(args[1]).intValue();
      }

      dispatchRexxScripts(rexxCode, vargs, nrThreads);
      System.out.println("--- Java/"+threadName+": after dispatchRexxScripts(), in main()\n");
      System.out.flush();

    }
    catch (BSFException e)  // if something went wrong, show it
    {
      e.printStackTrace();  // show the strack trace
    }

    System.exit(0);     // rgf, 2009-04-13; was commented, but if using *oo.rex, then this needs to be executed
  }


  /** Creates number of desired threads, runs them.
   */
  public static void dispatchRexxScripts(String rexxCode, Vector args, int nrThreads) throws Exception
  {
      System.out.println("--- Java: arrived in dispatchRexxScripts(): nrThreads=["+nrThreads+"]");
      System.out.flush();

      String      tmpResult="";
      Throwable   tmpTh=null;
      String      tmpthreadName=null;
        // create arrays of needed size
      Thread               t[]=new Thread[nrThreads];
      ThreadedJNIRexxStart r[]=new ThreadedJNIRexxStart[nrThreads];

      System.out.println("--> Java: creating and running threads...");
      System.out.flush();
      for (int i=0;i<nrThreads; i++)
      {
          r[i]=new ThreadedJNIRexxStart(rexxCode, args);
          t[i]=new Thread(r[i]);             // create thread
          r[i].setThreadName(t[i].getName());
          t[i].start();               // start execution on own thread
          System.out.println("--> Java: just started thread ["+r[i].getThreadName()+"]...");
          System.out.flush();
          // Thread.currentThread().sleep(250);  // sleep given msec before dispatching the next thread
      }

      // all are finished
      System.out.println("--> Java: joining all threads...");
      System.out.flush();
      for (int i=0;i<nrThreads; i++)        // join all threads
      {
          t[i].join();
      }

        // all are finished
      System.out.println("--> Java: all ["+nrThreads+"] threads ended, displaying all results...");
      System.out.flush();
      for (int i=0;i<nrThreads; i++)        // join all threads
      {
          if (r[i].getThrowable()!=null)
          {
              System.out.println("  * Java: # "+i+", exception: ["+r[i].getThrowable()+"]");
              System.out.flush();
          }
          else
          {
              System.out.println("  * Java: # "+i+", Rexx result=["+r[i].getResult()+"]");
              System.out.flush();
          }
      }

      System.out.println("--> Java: dispatchRexxScripts() ended, returning to caller...");
      System.out.flush();
  }





        // Runnable inner class
    static private class ThreadedJNIRexxStart implements Runnable
    {
        private Throwable   th        =null;
        private String      result    =null;    // result from ooRexx

        private String      rexxCode  =null;
        private Vector      args      =null;
        private String      threadName="n/a";

        ThreadedJNIRexxStart(String rexxCode, Vector args) {
            this.rexxCode   = rexxCode  ;
            this.args       = new Vector(args)      ;
        }


        public void run ()  // invoke ooRexx on its own Java thread
        {
            try {
                    // get a BSFManager instance
                  BSFManager mgr  = new BSFManager ();
                    // load the Rexx engine for this BSFManager instance
                  BSFEngine  rexx = mgr.loadScriptingEngine("rexx");
                    // a String that contains Rexx code (may be read from file)

                  String threadName=Thread.currentThread().getName();
                  System.out.println("--> Java: ThreadedJNIRexxStart.run(): BEFORE APPLY threadName=["+threadName+"]: mgr=["+mgr+"], engine=["+rexx+"]");
                  System.out.flush();


                    // execute the Rexx script
                  result=(String) rexx.apply ("rexx", 0, 0,
                                     (Object) this.rexxCode.replaceAll("@threadName", (""+this.threadName)),
                                     null,
                                     this.args);

                  System.out.println("--> Java: ThreadedJNIRexxStart.run(): AFTER  APPLY threadName=["+threadName+"]: mgr=["+mgr+"], engine=["+rexx+"]; result=["+result+"]");
                  System.out.flush();
            }
            catch (Throwable th) {
                this.th=th;         // save Throwable
            }
        }

        public String getResult() {    // getter
            return result;
        }

        public Throwable getThrowable() {   // getter
            return th;
        }

        public String getThreadName() {     // getter
            return threadName;
        }
        public void setThreadName(String threadName) {   // setter
            this.threadName=threadName;
            this.args.set(0, threadName);   // set threadName in argument vector
        }
    }
}
